@@ -1,12 +1,21 @@
-{-# LANGUAGE FlexibleInstances, OverlappingInstances, TypeSynonymInstances,
+{-# LANGUAGE DeriveGeneric #-}
+{-# LANGUAGE CPP, FlexibleInstances, TypeSynonymInstances,
     OverloadedStrings #-}
 
+#if __GLASGOW_HASKELL__ < 710
+{-# LANGUAGE OverlappingInstances #-}
+#endif
+
 module Database.Redis.Types where
 
+#if __GLASGOW_HASKELL__ < 710
 import Control.Applicative
+#endif
+import Control.DeepSeq
 import Data.ByteString.Char8 (ByteString, pack)
-import qualified Data.ByteString.Lex.Fractional as F (readSigned, readDecimal)
+import qualified Data.ByteString.Lex.Fractional as F (readSigned, readExponential)
 import qualified Data.ByteString.Lex.Integral as I (readSigned, readDecimal)
+import GHC.Generics
 
 import Database.Redis.Protocol
 
@@ -36,9 +45,11 @@ instance RedisArg Double where
 -- RedisResult instances
 --
 data Status = Ok | Pong | Status ByteString
-    deriving (Show, Eq)
+    deriving (Show, Eq, Generic)
 
-data RedisType = None | String | Hash | List | Set | ZSet
+instance NFData Status
+
+data RedisType = None | String | Hash | List | Set | ZSet
     deriving (Show, Eq)
 
 instance RedisResult Reply where
@@ -55,7 +66,7 @@ instance RedisResult Integer where
         maybe (Left r) (Right . fst) . I.readSigned I.readDecimal =<< decode r
 
 instance RedisResult Double where
-    decode r = maybe (Left r) (Right . fst) . F.readSigned F.readDecimal =<< decode r
+    decode r = maybe (Left r) (Right . fst) . F.readSigned F.readExponential =<< decode r
 
 instance RedisResult Status where
     decode (SingleLine s) = Right $ case s of
@@ -86,7 +97,11 @@ instance (RedisResult a) => RedisResult (Maybe a) where
     decode (MultiBulk Nothing) = Right Nothing
     decode r                   = Just <$> decode r
 
-instance (RedisResult a) => RedisResult [a] where
+instance
+#if __GLASGOW_HASKELL__ >= 710
+    {-# OVERLAPPABLE #-}
+#endif
+    (RedisResult a) => RedisResult [a] where
     decode (MultiBulk (Just rs)) = mapM decode rs
     decode r                     = Left r
  
