@@ -1,10 +1,11 @@
-{-# LANGUAGE CPP, OverloadedStrings, RecordWildCards #-}
+{-# LANGUAGE CPP, OverloadedStrings, RecordWildCards, LambdaCase #-}
 module Main (main) where
 
 #if __GLASGOW_HASKELL__ < 710
 import Control.Applicative
 import Data.Monoid (mappend)
 #endif
+import qualified Control.Concurrent.Async as Async
 import Control.Exception (try)
 import Control.Concurrent
 import Control.Monad
@@ -12,7 +13,6 @@ import Control.Monad.Trans
 import qualified Data.List as L
 import Data.Time
 import Data.Time.Clock.POSIX
-import SlaveThread (fork)
 import qualified Test.Framework as Test (Test, defaultMain)
 import qualified Test.Framework.Providers.HUnit as Test (testCase)
 import qualified Test.HUnit as HUnit
@@ -84,7 +84,9 @@ testConstantSpacePipelining = testCase "constant-space pipelining" $ do
 
 testForceErrorReply :: Test
 testForceErrorReply = testCase "force error reply" $ do
-    Right _ <- set "key" "value"
+    set "key" "value" >>= \case
+      Left _ -> error "impossible"
+      _ -> return ()
     -- key is not a hash -> wrong kind of value
     reply <- hkeys "key"
     assert $ case reply of
@@ -111,13 +113,14 @@ testEvalReplies :: Test
 testEvalReplies conn = testCase "eval unused replies" go conn
   where
     go = do
-        _ignored <- set "key" "value"
-        (liftIO $ do
-            threadDelay $ 10^(5::Int)
-            mvar <- newEmptyMVar
-            _ <- fork $ runRedis conn (get "key") >>= putMVar mvar
-            takeMVar mvar)
-          >>=? Just "value"
+      _ignored <- set "key" "value"
+      (liftIO $ do
+         threadDelay $ 10 ^ (5 :: Int)
+         mvar <- newEmptyMVar
+         _ <-
+           (Async.wait =<< Async.async (runRedis conn (get "key"))) >>= putMVar mvar
+         takeMVar mvar) >>=?
+        Just "value"
 
 ------------------------------------------------------------------------------
 -- Keys
@@ -136,17 +139,23 @@ testKeys = testCase "keys" $ do
     select 13             >>=? Ok
     expire "key" 1        >>=? True
     pexpire "key" 1000    >>=? True
-    Right t <- ttl "key"
-    assert $ t `elem` [0..1]
-    Right pt <- pttl "key"
-    assert $ pt `elem` [990..1000]
-    persist "key"         >>=? True
-    Right s <- dump "key"
-    restore "key'" 0 s    >>=? Ok
-    rename "key" "key'"   >>=? Ok
-    renamenx "key'" "key" >>=? True
-    del ["key"]           >>=? 1
-    select 0              >>=? Ok
+    ttl "key" >>= \case
+      Left _ -> error "error"
+      Right t -> do
+        assert $ t `elem` [0..1]
+        pttl "key" >>= \case
+          Left _ -> error "error"
+          Right pt -> do
+            assert $ pt `elem` [990..1000]
+            persist "key"         >>=? True
+            dump "key" >>= \case
+              Left _ -> error "impossible"
+              Right s -> do
+                restore "key'" 0 s    >>=? Ok
+                rename "key" "key'"   >>=? Ok
+                renamenx "key'" "key" >>=? True
+                del ["key"]           >>=? 1
+                select 0              >>=? Ok
 
 testExpireAt :: Test
 testExpireAt = testCase "expireat" $ do
@@ -161,14 +170,16 @@ testSort = testCase "sort" $ do
     lpush "ids"     ["1","2","3"]                >>=? 3
     sort "ids" defaultSortOpts                   >>=? ["1","2","3"]
     sortStore "ids" "anotherKey" defaultSortOpts >>=? 3
-    Right _ <- mset
+    mset
          [("weight_1","1")
          ,("weight_2","2")
          ,("weight_3","3")
          ,("object_1","foo")
          ,("object_2","bar")
          ,("object_3","baz")
-         ]
+         ] >>= \case
+      Left _ -> error "error"
+      _ -> return ()
     let opts = defaultSortOpts { sortOrder = Desc, sortAlpha = True
                                , sortLimit = (1,2)
                                , sortBy    = Just "weight_*"
@@ -195,7 +206,9 @@ testObject :: Test
 testObject = testCase "object" $ do
     set "key" "value"    >>=? Ok
     objectRefcount "key" >>=? 1
-    Right _ <- objectEncoding "key"
+    objectEncoding "key" >>= \case
+      Left _ -> error "error"
+      _ -> return ()
     objectIdletime "key" >>=? 0
 
 ------------------------------------------------------------------------------
@@ -357,8 +370,12 @@ testZSets = testCase "sorted sets" $ do
 
 testZStore :: Test
 testZStore = testCase "zunionstore/zinterstore" $ do
-    Right _ <- zadd "k1" [(1, "v1"), (2, "v2")]
-    Right _ <- zadd "k2" [(2, "v2"), (3, "v3")]
+    zadd "k1" [(1, "v1"), (2, "v2")] >>= \case
+      Left _ -> error "error"
+      _ -> return ()
+    zadd "k2" [(2, "v2"), (3, "v3")] >>= \case
+      Left _ -> error "error"
+      _ -> return ()
     zinterstore "newkey" ["k1","k2"] Sum                >>=? 1
     zinterstoreWeights "newkey" [("k1",1),("k2",2)] Max >>=? 1
     zunionstore "newkey" ["k1","k2"] Sum                >>=? 3
@@ -371,17 +388,29 @@ testZStore = testCase "zunionstore/zinterstore" $ do
 testHyperLogLog :: Test
 testHyperLogLog = testCase "hyperloglog" $ do
   -- test creation
-  Right _ <- pfadd "hll1" ["a"]
+  pfadd "hll1" ["a"] >>= \case
+      Left _ -> error "error"
+      _ -> return ()
   pfcount ["hll1"] >>=? 1
   -- test cardinality
-  Right _ <- pfadd "hll1" ["a"]
+  pfadd "hll1" ["a"] >>= \case
+      Left _ -> error "error"
+      _ -> return ()
   pfcount ["hll1"] >>=? 1
-  Right _ <- pfadd "hll1" ["b", "c", "foo", "bar"]
+  pfadd "hll1" ["b", "c", "foo", "bar"] >>= \case
+      Left _ -> error "error"
+      _ -> return ()
   pfcount ["hll1"] >>=? 5
   -- test merge
-  Right _ <- pfadd "hll2" ["1", "2", "3"]
-  Right _ <- pfadd "hll3" ["4", "5", "6"]
-  Right _ <- pfmerge "hll4" ["hll2", "hll3"]
+  pfadd "hll2" ["1", "2", "3"] >>= \case
+      Left _ -> error "error"
+      _ -> return ()
+  pfadd "hll3" ["4", "5", "6"] >>= \case
+      Left _ -> error "error"
+      _ -> return ()
+  pfmerge "hll4" ["hll2", "hll3"] >>= \case
+      Left _ -> error "error"
+      _ -> return ()
   pfcount ["hll4"] >>=? 6
   -- test union cardinality
   pfcount ["hll2", "hll3"] >>=? 6
@@ -394,7 +423,7 @@ testPubSub conn = testCase "pubSub" go conn
   where
     go = do
         -- producer
-        _ <- liftIO $ fork $ do
+        asyncProducer <- liftIO $ Async.async $ do
             runRedis conn $ do
                 let t = 10^(5 :: Int)
                 liftIO $ threadDelay t
@@ -414,6 +443,8 @@ testPubSub conn = testCase "pubSub" go conn
         pubSub (subscribe [] `mappend` psubscribe []) $ \_ -> do
             liftIO $ HUnit.assertFailure "no subs: should return immediately"
             undefined
+        liftIO $ Async.wait asyncProducer
+
 
 ------------------------------------------------------------------------------
 -- Transaction
@@ -422,8 +453,12 @@ testTransaction :: Test
 testTransaction = testCase "transaction" $ do
     watch ["k1", "k2"] >>=? Ok
     unwatch            >>=? Ok
-    Right _ <- set "foo" "foo"
-    Right _ <- set "bar" "bar"
+    set "foo" "foo" >>= \case
+      Left _ -> error "error"
+      _ -> return ()
+    set "bar" "bar" >>= \case
+      Left _ -> error "error"
+      _ -> return ()
     foobar <- multiExec $ do
         foo <- get "foo"
         bar <- get "bar"
@@ -440,24 +475,29 @@ testScripting conn = testCase "scripting" go conn
     go = do
         let script    = "return {false, 42}"
             scriptRes = (False, 42 :: Integer)
-        Right scriptHash <- scriptLoad script
-        eval script [] []                       >>=? scriptRes
-        evalsha scriptHash [] []                >>=? scriptRes
-        scriptExists [scriptHash, "notAScript"] >>=? [True, False]
-        scriptFlush                             >>=? Ok
-        -- start long running script from another client
-        configSet "lua-time-limit" "100"        >>=? Ok
-        evalFinished <- liftIO newEmptyMVar
-        void $ liftIO $ fork $ runRedis conn $ do
-            -- we must pattern match to block the thread
-            Left _ <- eval "while true do end" [] []
-                :: Redis (Either Reply Integer)
-            liftIO (putMVar evalFinished ())
+        scriptLoad script >>= \case
+          Left _ -> error "error"
+          Right scriptHash -> do
+            eval script [] []                       >>=? scriptRes
+            evalsha scriptHash [] []                >>=? scriptRes
+            scriptExists [scriptHash, "notAScript"] >>=? [True, False]
+            scriptFlush                             >>=? Ok
+            -- start long running script from another client
+            configSet "lua-time-limit" "100"        >>=? Ok
+            evalFinished <- liftIO newEmptyMVar
+            asyncScripting <- liftIO $ Async.async $ runRedis conn $ do
+                -- we must pattern match to block the thread
+                (eval "while true do end" [] []
+                    :: Redis (Either Reply Integer)) >>= \case
+                    Left _ -> return ()
+                    _ -> error "impossible"
+                liftIO (putMVar evalFinished ())
+                return ()
+            liftIO (threadDelay 500000) -- 0.5s
+            scriptKill                              >>=? Ok
+            () <- liftIO (takeMVar evalFinished)
+            liftIO $ Async.wait asyncScripting
             return ()
-        liftIO (threadDelay 500000) -- 0.5s
-        scriptKill                              >>=? Ok
-        () <- liftIO (takeMVar evalFinished)
-        return ()
 
 ------------------------------------------------------------------------------
 -- Connection
@@ -529,17 +569,23 @@ testsServer =
 
 testServer :: Test
 testServer = testCase "server" $ do
-    Right (_,_) <- time
+    time >>= \case
+      Right (_,_) -> return ()
+      Left _ -> error "error"
     slaveof "no" "one" >>=? Ok
     return ()
 
 testBgrewriteaof :: Test
 testBgrewriteaof = testCase "bgrewriteaof/bgsave/save" $ do
     save >>=? Ok
-    Right (Status _) <- bgsave
+    bgsave >>= \case
+      Right (Status _) -> return ()
+      _ -> error "error"
     -- Redis needs time to finish the bgsave
     liftIO $ threadDelay (10^(5 :: Int))
-    Right (Status _) <- bgrewriteaof
+    bgrewriteaof >>= \case
+      Right (Status _) -> return ()
+      _ -> error "error"
     return ()
 
 testConfig :: Test
@@ -556,8 +602,12 @@ testFlushall = testCase "flushall/flushdb" $ do
 
 testInfo :: Test
 testInfo = testCase "info/lastsave/dbsize" $ do
-    Right _ <- info
-    Right _ <- lastsave
+    info >>= \case
+      Left _ -> error "error"
+      _ -> return ()
+    lastsave >>= \case
+      Left _ -> error "error"
+      _ -> return ()
     dbsize          >>=? 0
     configResetstat >>=? Ok
 
@@ -570,7 +620,9 @@ testSlowlog = testCase "slowlog" $ do
 testDebugObject :: Test
 testDebugObject = testCase "debugObject/debugSegfault" $ do
     set "key" "value" >>=? Ok
-    Right _ <- debugObject "key"
+    debugObject "key" >>= \case
+      Left _ -> error "error"
+      _ -> return ()
     return ()
 
 testScans :: Test
@@ -666,15 +718,37 @@ testXpending = testCase "xpending" $ do
         Right bad -> HUnit.assertFailure $ "Unexpectedly got " ++ show bad
 
 testXClaim ::Test
-testXClaim = testCase "xclaim" $ do
-    xadd "somestream" "121" [("key1", "value1")]
-    xadd "somestream" "122" [("key2", "value2")]
-    xgroupCreate "somestream" "somegroup" "0"
-    xreadGroupOpts "somegroup" "consumer1" [("somestream", "0")] (defaultXreadOpts { recordCount = Just 2})
-    xclaim "somestream" "somegroup" "consumer2" 0 defaultXClaimOpts ["121-0"] >>=? [
-        StreamsRecord{recordId = "121-0", keyValues = [("key1", "value1")]}
+testXClaim =
+  testCase "xclaim" $ do
+    xadd "somestream" "121" [("key1", "value1")] >>=? "121-0"
+    xadd "somestream" "122" [("key2", "value2")] >>=? "122-0"
+    xgroupCreate "somestream" "somegroup" "0" >>=? Ok
+    xreadGroupOpts
+      "somegroup"
+      "consumer1"
+      [("somestream", "0")]
+      (defaultXreadOpts {recordCount = Just 2}) >>=?
+      Just
+        [ XReadResponse
+            { stream = "somestream"
+            , records =
+                [ StreamsRecord
+                    {recordId = "121-0", keyValues = [("key1", "value1")]}
+                , StreamsRecord
+                    {recordId = "122-0", keyValues = [("key2", "value2")]}
+                ]
+            }
         ]
-    xclaimJustIds "somestream" "somegroup" "consumer2" 0 defaultXClaimOpts ["122-0"] >>=? ["122-0"]
+    xclaim "somestream" "somegroup" "consumer2" 0 defaultXClaimOpts ["121-0"] >>=?
+      [StreamsRecord {recordId = "121-0", keyValues = [("key1", "value1")]}]
+    xclaimJustIds
+      "somestream"
+      "somegroup"
+      "consumer2"
+      0
+      defaultXClaimOpts
+      ["122-0"] >>=?
+      ["122-0"]
 
 testXInfo ::Test
 testXInfo = testCase "xinfo" $ do
